/* adc_logger.c
 * command line program to read values from the TS7200 ADC
 * Jim Jackson
 */
/*
 * Copyright (C) 2004 Jim Jackson           jj@franjam.org.uk
 */
/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program - see the file COPYING; if not, write to 
 *  the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, 
 *  MA 02139, USA.
 */
/*
 *  Date     Vers   Comment
 *  -----    -----  ---------------
 *  02Oct04  1      Initial Program Built from generic filter.c
 *  08Oct04  1.1    split off the ADC functions in ts7200io.c
 *  23Nov04  2.0    added individual range support and tidied up
 *  27Dec04  2.1    corrected period count to adjust for 10msec drift
 *                  (on machines with HZ=100) when doing repetitive readings
 *                  If HZ!=100 this needs fixing - see setting of adjperiod 
 *                  Also fixed an error with erroneous timestamp.
 *                  Also added '-z' to do repetitive readings without a 
 *                  usleep()
 *                  aprox 20,000 readings per sec on my 166MHz TS7200
 *  24Apr05  2.2    Added -T option to timestamp readings with time of day
 *   	            in hh:mm:ss.sss format
 *  24Jul06  2.3    Added O_SYNC ro open of /dev/mem to forces none
 *                  caching of memory accesses
 *                  
 */

/***MANPAGE***/
/*
adc_logger:

see file adc_logger.txt

 */

/***INCLUDES***/
/* includes files 
 */

#include <stdio.h>
/* for SYSV variants .... */
/* #include <string.h> */
#include <strings.h>
#include <errno.h>
#include <fcntl.h>
#include <signal.h>
#include <ctype.h>
#include <math.h>
#include <sys/time.h>
#include "ts7200io.h"

/***DEFINES***/
/* defines
 */

#define VERSION "2.2"

#define PVERSION(A) fprintf(stderr,"%s version %s\n",sys,A)

/* OUTFILEMODE  mode to use to open the default output file
 *  set here to "a" for append - alter to "w" for rewrite
 */

#define OUTFILEMODE "a"

#define DEF_RANGE (3)

#define EFAIL -1

#define TRUE 1
#define FALSE 0
#define nl puts("")

#define chkword(a,b) ((n>=a)&&(strncmp(p,b,n)==0))
#define index(A,B) strchr(A,B)
#define rindex(A,B) strrchr(A,B)

/***GLOBALS***/
/* global variables
 */

char *outfile;                         /* specified output file -o   */
char *sys;
int debug;                       /* set if the -d flag is used */
int vflg;                        /* set if -v flag used        */
int timestamp;                   /* -t add timestamp to each line */
int timeofday;			 /* -T add timeofday timestamp to line */
int zoomread;                    /* -z flag */
int nf;                          /* -nf flag */
int raw;			 /* -n output raw byte counts, do not convert
				       to a decimal voltage    */

struct timeval start,tm;
struct timezone tz;

/***DECLARATIONS***/
/* declare non-int functions 
 */

char *delnl(),*strupper(),*reads(),*strchr(),*strrchr(),*sindex(),
     *sindex_nc(),*tab(),*ctime();
FILE *fopen(),*mfopen();

/***MAIN***/
/* main - for program  adc_logger
 *        Simple command line program to read from the TS7200 ADC
 */

main(argc,argv)
int argc;
char **argv;

{ int i,j,n,t,t2,st;
  char fname[132],*p,*s;
  FILE *fpr,*fpw;
  int chan[16],range[16],maxchan,count,period,v,defrange;
  int adjperiod;

  argv[argc]=NULL;
  sys=*argv++; argc--;
  if ((p=rindex(sys,'/'))!=NULL) { sys=p+1; }

  nf=raw=timestamp=timeofday=debug=vflg=zoomread=FALSE;
  outfile=NULL;
  count=1;
  period=1000;
  defrange=DEF_RANGE;

  /* put global and local var. init. here */

  while (argc && **argv=='-') {          /* all flags and options must come */
    n=strlen(p=(*argv++)+1); argc--;     /* before paramters                */
    if (chkword(1,"output")) {      /* e.g. string option              */
      if (argc) {                        /* output file name           */
	outfile=*argv++; argc--;
      }
    } else if (chkword(1,"count")) {      
      if (argc) {                     
	count=atoi(*argv++); argc--;
      }
    } else if (chkword(1,"range")) {      
      if (argc) {                     
	defrange=atoi(*argv++); argc--;
      }
    } else if (chkword(1,"period")) { 
      if (argc) {                     
	period=atoi(*argv++); argc--;
      }
      zoomread=FALSE;
    } else {                            /* check for single char. flags    */
      for (; *p; p++) {
	if (*p=='h') exit(help(EFAIL));
	else if (*p=='t') { timestamp=TRUE; timeofday=FALSE; }
	else if (*p=='T') { timeofday=TRUE; timestamp=TRUE; }
	else if (*p=='n') raw=TRUE;
	else if (*p=='d') debug=TRUE;
	else if (*p=='v') vflg=TRUE;
	else if (*p=='z') zoomread=TRUE;
	else {
	  *fname='-'; *(fname+1)=*p; *(fname+2)=0;
	  exit(help(err_rpt(EINVAL,fname)));
	}
      }
    }
  }

  if ( defrange<0 || defrange>3 ) {
    exit(err_rpt(EINVAL,"range must be 0-3"));
  }
	
  if (argc<1 || argc>16 ) {
    exit(help(err_rpt(EINVAL,"Wrong number of parameters.")));
  }

  for (i=0; i<16 && argc; i++) {
    chan[i]=strtol(*argv++,&s,10); argc--;
    if (*s==',') {
      range[i]=atoi(++s);
    } else {
      range[i]=defrange;
    }
    if ( chan[i]<0 || chan[i]>7 ) {
      exit(err_rpt(EINVAL,"channel must be in range 0-7"));
    }
    if ( range[i]<0 || range[i]>7 ) {
      exit(err_rpt(EINVAL,"range must be in range 0-3"));
    }
  }
  maxchan=i;

  fpw=stdout; p="stdout";
  if (outfile!=NULL && (fpw=fopen((p=outfile),OUTFILEMODE))==NULL) {
    exit(err_rpt(errno,outfile));
  }
  outfile=p;

  strcpy(fname,"stdin");

  if (vflg) {
    PVERSION(VERSION);
    fprintf(stderr,"Writing to \"%s\"\n",p);
    fprintf(stderr,"Default Range set to %d\n",defrange);
    fprintf(stderr,"Read channels,ranges: %d,%d",chan[0],range[0]);
    for (i=1; i<maxchan; i++) {
      fprintf(stderr," %d,%d",chan[i],range[i]);
    }
    fprintf(stderr,"\nreading %d readings - ",count);
    if (zoomread)
      fprintf(stderr,"as fast a possible\n");
    else
      fprintf(stderr,"one every %d.%03d secs\n",period/1000,period%1000);
  }

  fflush(stderr);
  
  if (st=adc_init()) {
    exit(err_rpt(st,"Could Not initialise ADC or ADC hardware not present."));
  }
  
  if (vflg) {
    fprintf(stderr,"ADC initialised \n");
  }
  fflush(stderr);

  if (period>=10) { 
    adjperiod=period-10; 
  } else {
    adjperiod=0;
  }

  gettimeofday(&start, &tz);
  
  for ( i=0; i<count; i++) {
    if (timestamp) { 
      gettimeofday(&tm, &tz); 
      if (timeofday) {
	t=tm.tv_sec%(24*60*60);
	t2=tm.tv_usec/1000;
	fprintf(fpw,"%02d:%02d:%02d.%03d ",t/(60*60),(t/60)%60,t%60,t2);
      } else {
	t=tm.tv_sec-start.tv_sec;
        t2=(tm.tv_usec-start.tv_usec)/1000;
        if (t2<0) { t2+=1000; t--; }
        fprintf(fpw,"%d.%03d ",t,t2);
      }
    }
    for ( j=0; j<maxchan; j++) {
      if (raw) {
	fprintf(fpw,"%d ",adc_read(chan[j],range[j]));
      } else {
	n=readvolt(chan[j],range[j]);
	fprintf(fpw,"%d.%03d ",n/1000,(abs(n))%1000);
      }
    }
    fprintf(fpw,"\n");
    fflush(fpw);
    if (!zoomread) usleep(1000*adjperiod);
  }
   
  exit(st);
}

/***READV***/
/* readvolt(c,r)   get a reading for channel c with range r
 *              and convert the reading to voltage depending on the 
 *              range selected - return a double
 */

readvolt(c,r)
int c,r;
{
  int v;
  int a;

  v=adc_read(c,r);
  if (v>(1<<16)) return(v);
  switch (r) {
   case 0: a=((v*50000/4096)+5)/10; break;
   case 1: a=((v*100000/4096)+5)/10; break;
   case 2: a=((v*100000/4096)+5)/10; break;
   case 3: a=((v*200000/4096)+5)/10; break;
   otherwise: a=(ECHRNG<<16);
  }
  return(a);
}

/***HELP***/
/* help(e)  print brief help message - return parameter given
 */

help(e)
int e;
{
  PVERSION(VERSION);
  puts("\n\
Usage: adc_logger [-[ntv]] [-c N] [-p T] [-r R] chan[,range] [chan[,range]...]\
\n");
  printf("\
Read the ADC for the channels and ranges specified and output the results\n\
coverted to voltage.\n\n\
 -n    do not convert to volts - output raw ADC values\n\
 -t    timestamp each line output\n\
 -v    verbose\n\
 -z    zoomread, do N readings as fast as possible - with no usleep()\n\
 -c N  do N successive measurements for specified channels\n\
 -p T  with a period of T millisecs between each measurement\n\
       default is 1000mS - period only accurate to 10mS\n\
 -o file   output data to file, or append to file if already exists\n\
 -r R  set default range for measurements - this default defaults to 3\n\
\n\
",sys);
  return(e);
}

/***ERR_RPT***/
/* err_rpt(err,msg)
 */

err_rpt(err,msg)
short int err;
char *msg;

{ 
  if (err) fprintf(stderr,"[%s] %s : %s\n",sys,strerror(err),msg);
  return(err);
}

